<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\CNABSantander;

class Registro1CNABSantander extends RegistroCNAB{

    const REGISTRO_1 = '1';
    const VERSAO_LAYOUT = '031';
    const TIPO_OPERACAO_CREDITO = 'C';
    const TIPO_SERVICO = '30';
    const FORMA_LANCAMENTO_CREDITOCONTA = '01';

    private $_estrutura;
    private $_codConvenioBanco;

    public function geraRegistro1(){
        $this->_geraRegistro();
    }

    private function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_1, CNABSantander::CODIGO_BANCO_SANTANDER, self::LOTE_SERVICO);

        // Bloco 4 | 009 | Tipo da Operacao. C (Credito).
        $this->atribuiDado(self::TIPO_OPERACAO_CREDITO, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 5 | 010 - 011 | Tipo de Servico.
        $this->atribuiDado(self::TIPO_SERVICO, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 6 | 012-013 | Forma de Lancamento. 01 (Credito em Conta Corrente).
        $this->atribuiDado(self::FORMA_LANCAMENTO_CREDITOCONTA, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 7 | 014-016 | Numero da Versao do Layout. 031.
        $this->atribuiDado(self::VERSAO_LAYOUT, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 8 | 017 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 9 | 018 | Tipo de Inscricao da Empresa. 1 = CPF / 2 = CNPJ.
        $this->atribuiDado($this->_estrutura->nrTpInscricao, 1);

        // Bloco 10 | 019-032 | Numero Inscricao da Empresa.
        $this->atribuiDado($this->_estrutura->nrInscricao, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //@todo deve vir da estrutura
        // Bloco 11 | 033-052 | Codigo do Convenio no Banco.
        // Codigo adotado pelo Banco para identificar o Contrato entre este e a Empresa Cliente.
        // Deve ser fornecido pelo Banco.
        $this->atribuiDado($this->_codConvenioBanco, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 12 | 053-057 | Agencia Mantenedora da Conta.
        // Agencia Mantenedora deve ser preenchida sem DV.
        $this->atribuiDado($this->_estrutura->agencia, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 13 | 058 | Digito Verificador da Agencia. Deve ficar em branco.
        $this->atribuiDado(parent::CARACTER_BRANCO, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 14 | 059-070 | Numero da Conta Corrente.
        // Conta Corrente deve ser informada sem DV, pois este deve ser preenchido na posicao 71.
        $this->atribuiDado(parent::retornaContaCorrenteSemDigito($this->_estrutura->contaCorrente), 12, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 15 | 071 | Digito Verificador da Conta.
        $this->atribuiDado(parent::retornaDigitoContaCorrente($this->_estrutura->contaCorrente), 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 16 | 072 | Digito Verificador da Agencia / Conta. Deve ficar em branco.
        $this->atribuiDado(parent::CARACTER_BRANCO, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 17 | 073-102 | Nome da Empresa (Razao Social). Obrigatorio.
        $this->atribuiDado($this->_estrutura->historico->getNmrazsocestrut(), 30, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 18 | 103-142 | Filler (Caracter branco).
        $this->atribuiDado(parent::CARACTER_BRANCO, 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 19 | 143-172 | Endereco (Opcional).
        $this->atribuiDado($this->_estrutura->endereco->getEndereco(), 30, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 20 | 173-177 | Numero (Opcional).
        $this->atribuiDado($this->_estrutura->endereco->getNumero(), 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 21 | 178-192 | Complemento do Endereco (Opcional).
        $this->atribuiDado($this->_estrutura->endereco->getComplemento(), 15, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 22 | 193-212 | Municipio (Opcional).
        $this->atribuiDado($this->_estrutura->endereco->municipio, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 23 | 213-217 | CEP (Opcional).
        $this->atribuiDado(parent::retornaCepSemComplemento($this->_estrutura->endereco->getNrcependereco()), 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 24 | 218-220 | Complemento do CEP (Opcional).
        $this->atribuiDado(parent::retornaComplementoCep($this->_estrutura->endereco->getNrcependereco()), 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 25 | 221-222 | UF (Opcional).
        $this->atribuiDado($this->_estrutura->endereco->getSgestado(), 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 26 | 223-230 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 27 | 231-240 | Ocorrencias para o Retorno.
        // Codigo adotado para identificar as ocorrencias detectadas no processamento.
        // Pode-se informar ate 5 ocorrencias simultaneamente, cada uma delas codificada com dois
        // digitos, conforme relacao das paginas 32 e 33.
        $this->atribuiDado(parent::CARACTER_BRANCO, 10, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    public function setEstrutura($estrutura)
    {
        $this->_estrutura = $estrutura;
    }

    public function setCodConvenioBanco($codConvenioBanco)
    {
        $this->_codConvenioBanco = $codConvenioBanco;
    }

}