<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardocarvalho
 * Date: 19/11/13
 * Time: 09:18
 * To change this template use File | Settings | File Templates.
 */

namespace HCM\Util\ModeloImportacao;

use Folha\Entities\Tabprojimport;
use Geral\Entities\Novocodigo;
use Importacao\Logic\Integracao\GeradorSQL;
use Importacao\Logic\Integracao\Schema\Coluna;
use Importacao\Logic\Integracao\Schema\Tabela;
use Importacao\Logic\Bases\ModeloImportacao\Migra;
use Importacao\Logic\Util\ParametrosJson;
use Importacao\Logic\Util\Util;

class Integra {
    const PATTERN_DADOS = '/([0-9]{2}+\/+[0-9]{2}+\/+[0-9]{2})|((?<q>["]).*?\k<q>)|\?|([0-9]+,[0-9]*)|(\b\w+\b)/';
    const PATTERN_DIGITOS = '(\d+)';
    const PATTERN_STRINGS = '(\b(\w(-?))+\b)';
    const PATTERN_DATA = '/([0-9]{2}+[\/|-]+[0-9]{2}+[\/|-]+[0-9]{2})/';

    const TABELA = 'ADD TABLE';
    const COLUNA = 'ADD FIELD';
    const NRORG = 'NRORG';
    const DTINCLUSAO = 'DTINCLUSAO';
    const AUTOINCREMENT = '@AUTOINCREMENT@';
    const ID_CHAVE_NRORG = 'ID_CHAVE_NRORG';
    const DE_PARA = 'DE_PARA';
    const ID_SIM = 'S';
    const ID_NAO = 'N';

    const TIPO_INTEIRO = 'NUMBER';
    const TIPO_STRING  = 'VARCHAR2';
    const TIPO_LOGICO  = 'CHAR';
    const TIPO_DECIMAL = 'FLOAT';
    const TIPO_DATE    = 'DATE';

    const JSON_PATH = "../../sample.json";

    private $nrorg;

    private $xls;
    private $xlsPath;
    /** @var  $tabela Tabela */
    private $tabela;
    private $Json;
    public $agrupamento;
    public $diretorio;
    private $keyincremental;
    private $all_tables = array();

    public function __construct($xlsFilePath, $nrorg, $agrupamento) {
        $this->xlsPath = $xlsFilePath;
        $xls = \PHPExcel_IOFactory::createReaderForFile($xlsFilePath);
        /** @var $xlsReader \PHPExcel */
        $xlsReader = $xls->load($xlsFilePath);

        $this->xls = $xlsReader->getActiveSheet();
        $this->nrorg = $nrorg;
        $this->agrupamento = $agrupamento;

        $this->Json = new ParametrosJson(self::JSON_PATH);
    }

    /**
     * @return \Migracao\Logic\Integracao\Schema\Tabela
     */
    public function getTabela()
    {
        return $this->tabela;
    }

    private function simplify($var)
    {
        $acentos = array(
            'a' => '/À|Á|Â|Ã|Ä|Å/',
            'a' => '/à|á|â|ã|ä|å/',
            'c' => '/Ç/',
            'c' => '/ç/',
            'e' => '/È|É|Ê|Ë/',
            'e' => '/è|é|ê|ë/',
            'i' => '/Ì|Í|Î|Ï/',
            'i' => '/ì|í|î|ï/',
            'n' => '/Ñ/',
            'n' => '/ñ/',
            'o' => '/Ò|Ó|Ô|Õ|Ö/',
            'o' => '/ò|ó|ô|õ|ö/',
            'u' => '/Ù|Ú|Û|Ü/',
            'u' => '/ù|ú|û|ü/',
            'y' => '/Ý/',
            'y' => '/ý|ÿ/',
            'a.' => '/ª/',
            'o.' => '/º/'
        );
        $var = preg_replace($acentos, array_keys($acentos), $var);
        $var = preg_replace("/[^a-zA-Z0-9\s]/", "", $var);
        $var = trim($var);
        $var = str_replace(' ', '_', $var);
        $var = strtoupper($var);
        return $var;
    }

    /**
     * Cria a tabela com os dados das colunas do cabeçalho do arquivo, definindo os tipos das colunas
     *
     * @param $nome
     *
     * @author leonardocarvalho 29/11/2013
     */
    public function criaTabela($nome) {

        $tabela = new Tabela($nome);
        $highestColumnIndex = \PHPExcel_Cell::columnIndexFromString($this->xls->getHighestColumn());

        // Capturar o cabecalho para criar as colunas
        for ($column = 0; $column < $highestColumnIndex; $column++) {
            $celula = $this->xls->getCellByColumnAndRow($column, 1);
            $tabela->addColuna(new Coluna($this->simplify($celula->getValue())));
        }

        $colunas = $tabela->getColunas();

        foreach ($colunas as $colunaLabel) {
            // Verifica se a coluna do excel existe no Json
            if ($this->compareColunas($colunaLabel->getNome())) {
                // Capturar os tipos das células
                for ($column = 0; $column < $highestColumnIndex; $column++) {
                    $this->defineTiposColunas($column, $tabela);
                }
                $this->tabela = $tabela;

            } else {
                echo "Descrição da coluna $column - ".$colunaLabel->getNome()." do arquivo não existe.";
//                die;
            }
        }

        // retorna as colunas extras do agrupamento existentes no Json
        $colunasExtras = $this->Json->getColunasExtras($this->agrupamento);
        foreach ($colunasExtras as $coluna) {
            $tabela->addColuna(new Coluna($coluna));
        }

    }


    /**
     * Extrai todos os dados de cada coluna existente no arquivo e grava um Array
     *
     * @author leonardocarvalho - 29/11/2013
     */
    public function extraiDadosTabela() {
//        $highestColumnIndex2 = \PHPExcel_Cell::columnIndexFromString($this->xls->getHighestColumn());

        $highestColumnIndex = count($this->tabela->getColunas());        
        // Array para guardar em cada possiçao o dado de cada coluna
        $valor = array();

        // Percorre as linhas do excel
        $this->percorreLinhasExcel($valor, $highestColumnIndex);
    }

    /**
     * Gera o arquivo .SQL com a criação da tabela e com os inserts dos dados extraidos do arquivo
     *
     * @param $nomeSistema
     * @param bool $geraInsert
     * @param bool $geraDrop
     *
     * @author leonardocarvalho 29/11/2013
     */
    public function criaArquivoSql(Tabela $regTabela, $nomeSistema, $geraInsert = true, $geraDrop = true) {

        // Verifica se existe o diretório para ser criado o arquivo, caso não exista é criado com o nome do agrupamento
        $pastaAgrupador = explode("-", $nomeSistema);

        $this->diretorio = Util::verificaDiretorio(dirname($this->xlsPath)."/".$this->nrorg."/".$pastaAgrupador[0]);

        $geradorSQL = new GeradorSQL('DD/MM/YYYY');
        $sql = "";

        $this->tabela->setNome("{$nomeSistema}_{$regTabela->getNome()}");

        $geradorSQL->setTabela($regTabela);

        // Gera os inserts dos registros da tabela
        if ($geraInsert) {
            $sql .= $geradorSQL->geraInsert();
        }

        // agrupa todos os registros de todas as tabelas no vetor all_tables
        $this->all_tables[$pastaAgrupador[1]] = $sql;

        // gera o arquivo sql com os registros de CADA tabela, ou seja, cada tabela tem um arquivo sql
        //file_put_contents("$this->diretorio/$nomeSistema.sql", ($sql));
    }

    /**
     * Procura na estrutura do Json se existe a coluna vinda do arquivo excel.
     *
     * @param null $colunasExcel
     * @return bool
     *
     * @author leonardocarvalho 03/12/2013
     */
    public function compareColunas($colunaExcel = null) {

        foreach ($this->Json->getColunasLabel() as $colunaLabel) {
            // Verifica se existe na estrutura do Json a coluna passada pelo Excel
            if ($colunaExcel === $colunaLabel) {
                return true;
            }
        }

        return false;

    }

    /**
     * Define os tipos das colunas
     *
     * @param $column
     * @param $tabela
     *
     * @author leonardocarvalho
     */
    public function defineTiposColunas($column, $tabela)
    {

        $celula = $this->xls->getCellByColumnAndRow($column, 2);
        $coluna = $tabela->getColuna($column);
        $format = $celula->getWorksheet()->getStyle($celula->getCoordinate())->getNumberFormat()->getFormatCode();

        // Verifica o formato e define o tipo para a base de dados
        if ($format == 'DD/MM/YYYY') {
            $tipo = Integra::TIPO_DATE;
            $tamanhoMaximo = null;
        } else if (is_int($celula->getValue())) {
            $tipo = Integra::TIPO_INTEIRO;
            $tamanhoMaximo = null;
        } else if (is_float($celula->getValue())) {
            $tipo = Integra::TIPO_DECIMAL;
            $tamanhoMaximo = 126;
        } else {
            $tipo = Integra::TIPO_STRING;
            $tamanhoMaximo = 60;
        }

        $coluna->setTipo($tipo);
        $coluna->setTamanhoMaximo($tamanhoMaximo);
    }

    /**
     * Percorre as linhas do excel alimentando um vetor com os registros
     *
     * @param $valor
     * @param $highestColumnIndex
     * @param $registros
     * @return array
     *
     * @author leonardocarvalho 04/12/2013
     */
    public function percorreLinhasExcel($valor, $highestColumnIndex)
    {

        // retorna as colunas extras do agrupamento existentes no Json
        $colunasExtras = $this->Json->getColunasExtras($this->agrupamento);

        $this->verificaDePara($colunasExtras);

        for ($row = 2; $row <= $this->xls->getHighestRow(); $row++) {

            // Percorre as colunas do excel
            for ($column = 0; $column < $highestColumnIndex; $column++) {
                
                // Verifica se a coluna do excel existe no Json
                if ($this->compareColunas($this->tabela->getColuna($column)->getNome())) {

                    if ($this->tabela->getColuna($column)->getNome() === self::NRORG) {
                        $vr = $this->nrorg;
                    } elseif ($this->tabela->getColuna($column)->getNome() === self::DTINCLUSAO) {
                        $vr = 'sysdate';
                    } elseif (array_key_exists($this->tabela->getColuna($column)->getNome(), $colunasExtras)
                              && $this->tabela->getColuna($column)->getNome() === self::ID_CHAVE_NRORG) {
//                        $vr = $colunasExtras[self::ID_CHAVE_NRORG];
                    } elseif (array_key_exists($this->tabela->getColuna($column)->getNome(), $colunasExtras)) {
                        $vr = $row - 1;
                    } else {
                        $vr = trim($this->xls->getCellByColumnAndRow($column/*- count($colunasExtras)*/, $row)->getValue());
                    }

                    if ($this->tabela->getColuna($column)->getTipo() == self::TIPO_DATE) {
                        $vr = (trim($this->xls->getCellByColumnAndRow($column, $row)->getFormattedValue())) ? : null;
                    }                    
                    // Cria array com os dados de cada coluna da linha
                    $valor[$this->tabela->getColuna($column)->getNome()] = $vr;
                }
            }         
            $this->tabela->addRow($valor);
        }
    }

    public function organizaColunasPorTabela() {
        $tabelasJson = $this->Json->getTabelasByAgrupamento($this->agrupamento);

        foreach ($tabelasJson as $nmTabela) {
            // Zera chave incremental
            $this->keyincremental = 0;
            // Cria objeto da classe Tabela, passando o nome da tabela
            $tabela = new Tabela($nmTabela);
            // Retorna as labels da tabela atual que estão no Json
            $colunasLabelsJson = $this->Json->getColunasLabelByTabela($nmTabela);

            $this->populaTabela($colunasLabelsJson, $nmTabela, $tabela);
            $this->adicionaColunas($colunasLabelsJson, $nmTabela, $tabela);

            $this->criaArquivoSql($tabela, $this->agrupamento."-".$nmTabela);
        }

        // Gera arquivo .SQL com todos os registros de todas as tabelas do agrupamento
        file_put_contents("$this->diretorio/$this->agrupamento.sql", ($this->all_tables));

    }

    /**
     * @param $colunasExtras
     */
    public function verificaDePara($colunasExtras)
    {
        if (array_key_exists(self::DE_PARA, $colunasExtras)) {
            $tabprojimport = Tabprojimport::getRegistrosByAgrupamento($this->nrorg, $this->agrupamento);
            if ($tabprojimport->getIddepara() === self::ID_NAO) {
                throw new \Exception("Necessário realizar 'DE-PARA' do agrupamento");
            }
        }
    }

    /**
     * @param $colunasLabelsJson
     * @param $nmTabela
     * @param $tabela
     * @return array
     */
    private function adicionaColunas($colunasLabelsJson, $nmTabela, Tabela $tabela)
    {
        foreach ($this->tabela->getColunas() as $coluna) {
            if (array_key_exists($coluna->getNome(), $colunasLabelsJson)) {
                $json = $this->Json->getJsonDecode();
                $nmColuna = $json[$this->agrupamento][$nmTabela][$coluna->getNome()];
                if ($nmColuna === self::AUTOINCREMENT) {
                    $tabela->addColuna(new \Importacao\Logic\Integracao\Schema\Coluna($coluna->getNome()));
                } else {
                    $tabela->addColuna(new \Importacao\Logic\Integracao\Schema\Coluna($nmColuna));
                }
            }
        }
        print_r($tabela->getColunas());die;
    }

    /**
     * @param $colunasLabelsJson
     * @param $nmTabela
     * @param $tabela
     */
    private function populaTabela($colunasLabelsJson, $nmTabela, Tabela $tabela)
    {        
        foreach ($this->tabela->getRows() as $row) {
            $valor = array();
            $cont = 1;            
            foreach ($row as $key => $coluna) {
                if (array_key_exists($key, $colunasLabelsJson)) {
                    $json = $this->Json->getJsonDecode();
                    $nmColuna = $json[$this->agrupamento][$nmTabela][$key];
                    if ($nmColuna === self::AUTOINCREMENT) {
                        if (array_key_exists($key, Util::retornaPrimaryKey($nmTabela))
                            || $key === Util::retornaPrimaryKey($nmTabela) || $cont === 1 && $key !== self::NRORG) {
                            $coluna = Util::novoCodigo($nmTabela) + $coluna;
                            if ($cont === 1){
                                if ($this->keyincremental != 0) {
                                    $coluna = $this->keyincremental + 1;
                                    $this->keyincremental = $coluna;
                                } else {
                                    $this->keyincremental = $coluna;
                                }
                            }
                        }
                        $valor[$key] = $coluna;
                    } else {
                        $valor[$nmColuna] = $coluna;
                    }
                }
                $cont++;
            }                                    

            $tabela->addRow($this->splitKey($valor));
        }
    }

    public function splitKey($valor) {
        foreach ($valor as $key => $value) {
            if (preg_match("/\,/", $key)) {
                $aux = explode(",", $key);
                foreach ($aux as $newIndex) {
                    $valor[$newIndex] = $value;
                }
                unset($valor[$key]);
            }
        } 
        return $valor; 
    }
}

function login($nmorganizacao, $cdfilial, $acesso, $senha) {
    $login = new \LoginLogicProdutos($nmorganizacao, $cdfilial, $acesso, $senha);

    \Ambiente::incluiVariaveisAmbiente($login);

    $parametros = array(
        'NRORG' => $login->getNrorg(),
        'CDOPERADOR' => $login->getCdoperador()
    );

    $fpa = new \FpaLoginLogic($parametros);
}

require '../../../../../../projects/mig/public/bootstrap.php';

login('teknisa', 9999, 'leo', '1');

\FPA::setNrorgTrabalho(1008);
require __DIR__.'../../../../../../../../setforms/setforms/lib/PHPExcel/PHPExcel.php';

$xlsPath = 'C:/Documents and Settings/paulopereira/Desktop/migracao_total/IMPORTACAO ESTRUTURA TOTAL.xls';

$integra = new Integra($xlsPath, \FPA::getNrorgTrabalho(), 'ESTRUTURA');
$integra->criaTabela('EXCEL');
$integra->extraiDadosTabela();
$integra->organizaColunasPorTabela();
// Util::executaSQL("$integra->diretorio/$integra->agrupamento.sql");